from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject, ImanFile, ImanRelation, Dataset, Item, ItemRevision
from typing import Dict, List
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcBaseObj
from dataclasses import dataclass


@dataclass
class FormInfo(TcBaseObj):
    """
    Input structure for createTwoDSnapshot
    
    :var name: Identifier that helps the client to track the object created.
    :var description: Description of form.
    :var formType: Type of form, typically set to "Vis_Snapshot_2D_Form".
    :var attributesMap: List of key value pairs that make up the forms data.
    :var saveDB: Whether data should be saved, typicaly set true.
    """
    name: str = ''
    description: str = ''
    formType: str = ''
    attributesMap: StringArrayMap = None
    saveDB: bool = False


@dataclass
class GRMRelationOutput(TcBaseObj):
    """
    This structure holds the association information between the created markup Dataset and the launched Teamcenter
    object on which markups were created.
    
    :var relation: A Teamcenter object referencing relation information between the created markup Dataset and the
    launched Teamcenter object on which markups were created.
    e.g., VisStructureContext, Datasets like DirectModel
    :var propertiesInfo: A map of key value pairs representing the attributes assigned to the relation connecting the
    created markup Dataset to the launched Teamcenter object by the client application.
    """
    relation: ImanRelation = None
    propertiesInfo: KeyValueMap = None


@dataclass
class GetLatestFileReadTicketsInfo(TcBaseObj):
    """
    This structure is used to find the latest version of the named reference to the specified Dataset.
    
    :var clientId: Identifier that helps the client track the object
    :var dataset: Dataset owning the named reference object.
    :var namedRef: The IMANFile named reference.
    :var originalFilename: The original filename. This is used to search for the latest named reference in the Dataset.
    """
    clientId: str = ''
    dataset: Dataset = None
    namedRef: ImanFile = None
    originalFilename: str = ''


@dataclass
class GetLatestFileReadTicketsOutput(TcBaseObj):
    """
    Output structure to hold the data returned from getLatestFileReadTickets
    
    :var clientId: Identifier that helps the client track the object. This is the same 'clientId' from the input data.
    :var namedRefUID: The UID of the resolved IMANFile named reference.
    :var originalFilename: The original filename. This is the original filename of the resolved named reference.
    :var readTicket: The read ticket for the IMANFile in the latest Dataset.
    """
    clientId: str = ''
    namedRefUID: str = ''
    originalFilename: str = ''
    readTicket: str = ''


@dataclass
class GetLatestFileReadTicketsResponse(TcBaseObj):
    """
    This structure holds the data returned from the 'getLatestFileReadTickets'.
    
    :var getLatestFileReadTicketsOutput: A vector containing the latest Dataset versions and the associated IMANFile
    including the read file ticket.
    :var serviceData: Standard 'ServiceData' member
    """
    getLatestFileReadTicketsOutput: List[GetLatestFileReadTicketsOutput] = ()
    serviceData: ServiceData = None


@dataclass
class IdInfo(TcBaseObj):
    """
    This structure holds the information about the objects that will be launched to the viewer.
    
    :var id: A required parameter that references the object to be launched. If needed, launched object will be
    resolved by the server to a launch able object.
    :var item: An optional object reference of the Item containing launch able object. If this is not known, the server
    will attempt to identify the parent if it can.
    :var itemRev: An optional object reference of the ItemRevision containing launchable object. If this is not known,
    the server will attempt to identify if it can.
    :var operation: An optional parameter references the type of launch action. This controls the action the viewer
    will perform when it opens the object. The actions supported are one of following: 'Open', 'Insert', 'Merge' or
    'Interop'.  'Open' will open the object in a new window.  'Insert' will insert the object into the current window
    that has focus.  'Merge' will attempt to merge a pruned product structure with one that is already open if it can. 
    'Interop' will present a dialog that lets the user select the launch action.
    :var idAdditionalInfo: An optional parameter referencing the additional information of launched objects in form of
    key/value pairs (if any).
    """
    id: BusinessObject = None
    item: Item = None
    itemRev: ItemRevision = None
    operation: str = ''
    idAdditionalInfo: KeyValueMap = None


@dataclass
class MarkupInput(TcBaseObj):
    """
    This structure holds the information related to creation of the markup Dataset , association information of the
    markup Dataset to the rest of the Teamcenter objects and the list of the files to be uploaded in the markup Dataset.
    
    :var clientId: (Required) Unique Identifier defined by user to track for each markup Dataset creation request.
    :var attachToLocation: (Optional) Teamcenter object reference where the created markup Dataset is referenced.
    :var createDatasetInfo: (Required) An object of type 'DatasetInfo' struct, containing markup Dataset creation
    information.
    :var baseObjInfo: (Required) An object of type 'BaseObjInfo' struct containing information of Teamcenter launched
    object on which markups are created and will be connected to the created markup Dataset.
    :var uploadData: (Required) A vector of objects of type 'NamedRefUploadOrUpdateInfo' struct where each object
    contains information specific to the file that will be uploaded to the markup Dataset.
    """
    clientId: str = ''
    attachToLocation: BusinessObject = None
    createDatasetInfo: DatasetInfo = None
    baseObjInfo: BaseObjInfo = None
    uploadData: List[NamedRefUploadOrUpdateInfo] = ()


@dataclass
class MarkupOutput(TcBaseObj):
    """
    This structure holds the information related to creation of the markup Dataset, association information of the
    markup Dataset to the rest of the Teamcenter objects and the list of the files uploaded in the markup Dataset.
    
    :var markupDatset: A Teamcenter object referencing the created markup Dataset.
    :var references: A vector of object of type 'NamedRefsInDataset' struct, each object containing information about
    the successfully uploaded files in the markup Dataset.
    :var attachRelation: A Teamcenter object referencing relation information between the created markup Dataset and
    the Teamcenter object in which it is referenced.
    :var relation: An object of type relation struct, containing information about the Teamcenter objects that the
    created markup Dataset is connected to.
    """
    markupDatset: Dataset = None
    references: List[NamedRefsInDataset] = ()
    attachRelation: ImanRelation = None
    relation: GRMRelationOutput = None


@dataclass
class MarkupResponse(TcBaseObj):
    """
    This structure is used to return information from the 'createMarkup' operation. This structure holds information
    about 'serviceData' (containing information about successfully created markup Dataset and if any updated Teamcenter
    object. It may also contain information if any of the failure to commit the files to the Dataset.  In case of
    failure to create session Dataset altogether, error messages are associated to 'clientId' identifier and are added
    to the error stack).  'MarkupOutputMap', contains information mapping the 'clientId' identifier to successfully
    created markup Dataset along with the list of named references in the created markup Dataset, and the Teamcenter
    objects it is connected to.
    
    :var markupInfo: A map associating the unique clientId identifier with the 'MarkupOutput' struct.
    :var serviceData: Created Dataset will be returned in the 'ServiceData'. Any failure will be returned with error
    message in the 'ServiceData' list of partial errors.
    """
    markupInfo: MarkupOutputMap = None
    serviceData: ServiceData = None


@dataclass
class MarkupUpdateInput(TcBaseObj):
    """
    This structure holds the information about the updated markup Dataset, (if any) association information of the
    markup Dataset to the Teamcenter object  (VisStructureContext), and the list of the files to be uploaded in the
    markup Dataset.
    
    :var clientId: (Required)  Unique Identifier defined by user to track for each markup Dataset creation request.
    :var markupDataset: (Required)  Teamcenter object referencing markup Dataset that needs to be updated.
    :var baseObjInfo: (Optional) Objects of type 'BaseObjInfo' struct containing information of Teamcenter object
    (VisStructureContext), on which markups are created and connected to the markup Dataset.
    
    NOTE: This parameter is relevant when markups are updated against VisStructureContext. 
    
    :var uploadData: (Required)  A vector of objects of type 'NamedRefUploadOrUpdateInfo' struct where each object
    contains information specific to the file that will be uploaded to the markup Dataset.
    """
    clientId: str = ''
    markupDataset: Dataset = None
    baseObjInfo: BaseObjInfo = None
    uploadData: List[NamedRefUploadOrUpdateInfo] = ()


@dataclass
class MarkupUpdateOutput(TcBaseObj):
    """
    This structure holds the information related of the updated markup Dataset, association information of the markup
    Dataset to the VisStructureContext and the list of the files uploaded in the markup Dataset.
    
    :var markupDatset: A Teamcenter object referencing the updated markup Dataset
    :var references: A vector of object of type 'NamedRefsInDataset' struct, each object containing information about
    the successfully uploaded files in the markup Dataset.
    :var relation: An object of type 'relation' struct, containing information about the VisStructureContext that the
    updated markup Dataset is connected to.
    """
    markupDatset: Dataset = None
    references: List[NamedRefsInDataset] = ()
    relation: GRMRelationOutput = None


@dataclass
class MarkupUpdateResponse(TcBaseObj):
    """
    This structure is used to return information from the 'updateMarkup' operation. This structure holds information
    about 'serviceData' (containing information about successfully updated markup Dataset. It may also contain
    information if any of the failure to commit the file to the Dataset. In case of failure to update Dataset
    altogether, error messages are associated to 'clientId' identifier and are added to the error stack).
    'MarkupUpdateOutputMap', contains information mapping the 'clientId' identifier to successfully updated markup
    Dataset along with the list of named references in the updated markup Dataset, and the Teamcenter object
    (VisStructureContext) it is connected to.
    
    :var markupInfo: A map associating the unique 'clientId' identifier with the 'MarkupUpdateOutput' struct.
    :var serviceData: Updated Dataset will be returned in the 'serviceData'. Any failure will be returned with error
    message in the 'serviceData' list of partial errors.
    """
    markupInfo: MarkupUpdateOutputMap = None
    serviceData: ServiceData = None


@dataclass
class MetaDataStampTicketsResponse(TcBaseObj):
    """
    Structure used for 'getMetaDataStamp' response.
    
    :var tickets: A vector of FMS transient file ticket corresponding to the ids in the input data.
    :var serviceData: Failures will be returned in the 'Servicedata', with the error message in the 'ServiceData' list
    of partial errors.
    """
    tickets: List[str] = ()
    serviceData: ServiceData = None


@dataclass
class NamedRefUploadOrUpdateInfo(TcBaseObj):
    """
    This structure holds the information related to the files to be uploaded in the created markup Dataset.
    
    :var clientFileId: (Required) Unique Identifier defined by user to track each files that needs to be uploaded in
    the markup Dataset.
    :var fileState: (Required) State of the file at the time of upload. Options allowed are 'NEW', 'UPDATE', 'DELETE',
    'RENAME', 'RENAME_UPDATE'.
    
    e.g.  If creating then it will be the 'NEW'. If updating Dataset files the state will be either 'UPDATE', 'DELETE',
    'RENAME' or 'RENAME_UPDATE'.
    :var fileName: (Required) The name of file to be uploaded in the markup Dataset. Filename with extension needs to
    be provided.
    :var refName: (Required) Named reference of the file e.g. Markup.
    :var fileTicket: (Required) FMS write ticket of the file to be used at the time of commit.
    :var newObject: (Optional) Teamcenter IMANFile object reference to be added in the created markup Dataset. 
    
    Example scenarios where the parameter is relevant:
    - Adding a file reference from another Dataset.
    - Updating the IMANFile object existing in the Dataset, referenced by 'existingObject' parameter, with the new
    IMANFile object, referenced by 'newObject' parameter in Dataset.
    
    
    :var existingObject: (Optional) Teamcenter IMANFile object reference in the created markup Dataset.
    
    Example scenarios where the parameter is relevant:
    - Updating existing IMANFile Object referenced by 'existingObject' parameter with the new IMANFile object
    referenced by 'newObject' parameter in Dataset.  
    - Deleting the existing IMANFile object referenced by 'existingObject' parameter.
    
    """
    clientFileId: str = ''
    fileState: str = ''
    fileName: str = ''
    refName: str = ''
    fileTicket: str = ''
    newObject: BusinessObject = None
    existingObject: BusinessObject = None


@dataclass
class NamedRefsInDataset(TcBaseObj):
    """
    This structure holds the information related to the list of the files to be uploaded in the markup Dataset.
    
    :var clientFileId: Unique Identifier defined by user to track each file that was successfully uploaded to the
    markup Dataset.
    
    Note: This identifier is provided by 'NamedRefUploadOrUpdateInfo' structure.
    
    :var uploadedFile: Uploaded Teamcenter IMANFile object reference in the markup Dataset.
    :var uploadedFilename: Uploaded Teamcenter IMANFile object filename in the markup Dataset.
    :var uploadedFileRefNames: Uploaded Teamcenter IMANFile object reference name in the markup Dataset e.g. Markup.
    """
    clientFileId: str = ''
    uploadedFile: ImanFile = None
    uploadedFilename: str = ''
    uploadedFileRefNames: str = ''


@dataclass
class BaseObjInfo(TcBaseObj):
    """
    This structure holds the information related to Teamcenter launched object on which markup Dataset is created and
    connected to.
    
    :var baseObj: (Required) The object referencing Teamcenter objects like Dataset or VisStructureContext on which
    markup Dataset is created and connected to.
    :var propertiesInfo: (Optional) A map of key value pairs to assign the properties to the relation connecting the
    created markup Dataset to the launched Teamcenter object.
    
    NOTE: This parameter is relevant when creating a markup Dataset against a VisStructureContext. 
    """
    baseObj: BusinessObject = None
    propertiesInfo: KeyValueMap = None


@dataclass
class SaveSessionOutput(TcBaseObj):
    """
    This structure holds the information related to the session file to be uploaded in the session Dataset.
    
    :var sessionModelDataset: Teamcenter object referencing updated Vis_Session Dataset.
    :var reference: An object of type 'NamedRefsInDataset' struct, containing information about the successfully
    uploaded session file in the Vis_Session Dataset.
    """
    sessionModelDataset: Dataset = None
    reference: NamedRefsInDataset = None


@dataclass
class SaveSessionResponse(TcBaseObj):
    """
    This structure is used to return information from the 'saveSession' operation. This structure holds information
    about 'serviceData' and 'SessionToSessionModelMap'.  The 'serviceData' contains information about successfully
    updated session Dataset. It may also contain information if any on a failure to commit the file to the Dataset. In
    case of failure to update session Dataset, error messages are associated to 'clientId' identifier and are added to
    the error stack. The map, 'SessionToSessionModelMap', contains information mapping 'clientId' identifier to
    successfully updated session Dataset along with the information corresponding to the file uploaded in the session
    Dataset.
    
    :var sessionToSessionModelInfo: A map associating the unique 'clientId' identifier with the 'SaveSessionOutput'
    struct.
    :var serviceData: Updated session Dataset will be returned in the 'serviceData'. Any failure will be returned with
    error message in the 'serviceData' list of partial errors.
    """
    sessionToSessionModelInfo: SessionToSessionModelMap = None
    serviceData: ServiceData = None


@dataclass
class ServerInfo(TcBaseObj):
    """
    This structure holds the basic information for Teamcenter Visualization to connect to the server.
    
    :var protocol: A required parameter referencing the protocol type for connection to the server. Use http for
    standard 4 tier connections, and iiop for 2 tier deployments.
    :var hostpath: A required parameter referencing the URL to connect to the server.
    :var servermode: A required parameter referencing the server mode that controls how the connection to the server is
    made: 2 for two tier. 4 for four tier.
    :var serverAdditionalInfo: An optional parameter referencing the additional information of server in form of
    key/value pairs (if any).
    """
    protocol: str = ''
    hostpath: str = ''
    servermode: int = 0
    serverAdditionalInfo: KeyValueMap = None


@dataclass
class SessionInfo(TcBaseObj):
    """
    This structure holds the information about the session information of the client application from where the launch
    operation was initiated.
    
    :var sessionDescriminator: Client/Server session discriminator to connect to existing specified session.  This
    allows the viewer application to share an existing server process session with the client that initiated the
    launch. If this is not specified, the viewer will present a login prompt.
    :var sessionAdditionalInfo: An optional parameter referencing the additional information of the session in form of
    key/value pairs (if any).
    """
    sessionDescriminator: str = ''
    sessionAdditionalInfo: KeyValueMap = None


@dataclass
class SessionModelInput(TcBaseObj):
    """
    This structure holds the information related to creation of the markup Dataset , association information of the
    markup Dataset to the rest of the Teamcenter objects and the list of the files to be uploaded in the markup Dataset.
    
    :var clientId: (Required) Unique Identifier defined by user to track each session (Vis_Session) Dataset creation
    request.
    :var attachToLocation: (Optional) Teamcenter object reference where the created session Dataset is referenced.
    :var createDatasetInfo: (Required) An object of type 'DatasetInfo' struct, containing session Dataset creation
    information.
    :var sessionModelBaseObjs: (Required) A vector of type 'BaseObjInfo' struct where each object contains information
    of Teamcenter launched object, on which session is created. These objects are then connected to the created session
    Dataset via relations.
    :var uploadData: (Required) A vector of objects of type 'NamedRefUploadOrUpdateInfo' struct where each object
    contains information specific to the file that will be uploaded to the markup Dataset.
    """
    clientId: str = ''
    attachToLocation: BusinessObject = None
    createDatasetInfo: DatasetInfo = None
    sessionModelBaseObjs: List[BaseObjInfo] = ()
    uploadData: List[NamedRefUploadOrUpdateInfo] = ()


@dataclass
class SessionModelOutput(TcBaseObj):
    """
    This structure holds the information related to creation of the session Dataset , association information of the
    session Dataset to the rest of the Teamcenter objects and the list of the files uploaded in the session Dataset.
    
    :var sessionModelDatset: A Teamcenter object referencing the created Vis_Session  Dataset.
    :var references: A vector of objects of type 'NamedRefsInDataset' struct, each object containing information about
    the successfully uploaded files in the session Dataset.
    :var attachRelation: A Teamcenter object referencing relation information between the created markup Dataset and
    the Teamcenter object in which it is referenced.
    :var relations: A vector of objects of type 'GRMRelationOutput' struct, containing information about the Teamcenter
    objects that the created session Dataset is connected to.
    """
    sessionModelDatset: Dataset = None
    references: List[NamedRefsInDataset] = ()
    attachRelation: ImanRelation = None
    relations: List[GRMRelationOutput] = ()


@dataclass
class SessionModelResponse(TcBaseObj):
    """
    This structure is used to return information from the 'createSessionModel' operation. This structure holds
    information about 'serviceData' (containing information about successfully created session Dataset and if any
    updated Teamcenter object. It may also contain information if any of the failure to commit the files to the
    Dataset. In case of failure to create session Dataset altogether, error messages are associated to 'clientId'
    identifier and are added to the error stack). 'SessionModelOutputMap', contains information mapping 'clientId'
    identifier to successfully created session Dataset along with the list of named references in the created session
    Dataset, and the Teamcenter objects it is connected to.
    
    :var sessionModelInfo: A map associating the unique 'clientId' identifier with the 'SessionModelOutput' struct.
    :var serviceData: Created session Dataset will be returned in the 'serviceData'. Any failure will be returned with
    error message in the 'serviceData' list of partial errors.
    """
    sessionModelInfo: SessionModelOutputMap = None
    serviceData: ServiceData = None


@dataclass
class SessionModelUpdateOutput(TcBaseObj):
    """
    This structure holds the information related to updated session Dataset, association information of the session
    Dataset to the rest of the Teamcenter objects and the list of the files uploaded in the session Dataset.
    
    :var sessionModelDataset: A Teamcenter object referencing the updated  Vis_Session  Dataset.
    :var references: A vector of objects of type 'NamedRefsInDataset' struct, each object containing information about
    the successfully uploaded files in the session Dataset.
    :var relations: A vector of objects of type 'GRMRelationOutput' struct, containing association information between
    the input the Teamcenter objects and the updated session Dataset.
    """
    sessionModelDataset: Dataset = None
    references: List[NamedRefsInDataset] = ()
    relations: List[GRMRelationOutput] = ()


@dataclass
class SessionModelUpdateResponse(TcBaseObj):
    """
    This structure is used to return information from the 'updateSessionModel' operation. This structure holds
    information about 'serviceData' (containing information about successfully updated session Dataset.  It may also
    contain information if any of the failure to commit the file to the Dataset. In case of failure to update session
    Dataset altogether, error messages are associated to 'clientId' identifier and are added to the error stack).
    'SessionModelUpdateOutputMap' contains information mapping 'clientId' identifier to successfully updated session
    Dataset along with the information corresponding to the file uploaded in the session Dataset and updated relation
    of the session Dataset to the input Teamcenter objects.
    
    :var sessionModelInfo: A map associating the unique 'clientId' identifier with the 'SessionModelUpdateOutput'
    struct.
    :var serviceData: Updated session Dataset will be returned in the 'serviceData'. Any failure will be returned with
    error message in the 'serviceData' list of partial errors.
    """
    sessionModelInfo: SessionModelUpdateOutputMap = None
    serviceData: ServiceData = None


@dataclass
class TwoDSnapshotInput(TcBaseObj):
    """
    Input structure for create snapshot datasets operation
    
    :var clientId: Identifier that helps the client to track the object created, optional.
    :var sourceItemRev: Name of the source item revision to which snapshot dataset will be attached.  This represents
    the ItemRevision that owns the base image.
    :var createDatasetInfo: A structure containing information about the Vis_Snapshot_2D_View_Data dataset including
    tool to open the dataset, dataset type, name, and description. The datasetType should be set to
    "Vis_Snapshot_2D_View_Data".
    :var namedRefInfo: A vector of structures each containing information about a named reference.  Should include the
    markups, viewport xml file, thumbnail, and asset image files along with the object references to the base image
    dataset and the source ItemRevision objects.
    :var createFormInfo: A structure containing information about form attribute information.
    """
    clientId: str = ''
    sourceItemRev: ItemRevision = None
    createDatasetInfo: DatasetInfo = None
    namedRefInfo: List[NamedRefUploadOrUpdateInfo] = ()
    createFormInfo: FormInfo = None


@dataclass
class UpdateSessionModelWithSessionInput(TcBaseObj):
    """
    This structure holds the information of Vis_Session Dataset that will be updated with the session file.
    
    :var clientId: (Required) Unique Identifier defined by user to track each session (Vis_Session) Dataset that will
    be updated.
    :var sessionModelDataset: (Required) Teamcenter object referencing Vis_Session Dataset that needs to be updated.
    :var uploadSessionFile: (Required) An object of type 'NamedRefUploadOrUpdateInfo' struct, containing information of
    the session file that will be uploaded into the Vis_Session Dataset.
    """
    clientId: str = ''
    sessionModelDataset: Dataset = None
    uploadSessionFile: NamedRefUploadOrUpdateInfo = None


@dataclass
class UpdateSesssionModel(TcBaseObj):
    """
    This structure holds the information about the updated session Dataset, (if any) association information of the
    session Dataset to the Teamcenter object (VisStructureContext), and the list of the files to be uploaded in the
    session Dataset.
    
    :var clientId: (Required) Unique Identifier defined by user to track for each markup Dataset creation request.
    :var sessionModelDataset: (Required) Teamcenter object referencing session Dataset that needs to be updated.
    :var sessionModelBaseObjs: (Optional) A vector of objects of type 'BaseObjInfo' struct , each containing
    information of Teamcenter objects included in session and connected to the session Dataset.
    
    NOTE: This parameter may be relevant when session contains new references.
    
    :var uploadData: (Required) A vector of objects of type 'NamedRefUploadOrUpdateInfo' struct where each object
    contains information specific to the file that will be uploaded to the session Dataset.
    """
    clientId: str = ''
    sessionModelDataset: Dataset = None
    sessionModelBaseObjs: List[BaseObjInfo] = ()
    uploadData: List[NamedRefUploadOrUpdateInfo] = ()


@dataclass
class UpdateTwoDSnapshotInput(TcBaseObj):
    """
    Input structure for update snapshot dataset operation
    
    :var clientId: Identifier that helps the client to track the object created, optional.
    :var snapshotDataset: A structure containing information about the Vis_Snapshot_2D_View_Data dataset such as type,
    name, and description.  The datasetType should be set to "Vis_Snapshot_2D_View_Data".
    :var namedRefInfo: A vector of structures  each containing information about named references.  Same structure used
    by the createTwoDSnapshot service.  Should include the markup layers, viewport xml file, thumbnail, and asset image
    files along with the object references to the base image dataset and the source ItemRevision objects.
    """
    clientId: str = ''
    snapshotDataset: Dataset = None
    namedRefInfo: List[NamedRefUploadOrUpdateInfo] = ()


@dataclass
class UpdateTwoDSnapshotOutput(TcBaseObj):
    """
    Output structure for update 2D Snapshot operation
    
    :var snapshotDataset: Dataset object reference that was created.
    :var references: List of named references associated with dataset.
    """
    snapshotDataset: Dataset = None
    references: List[NamedRefsInDataset] = ()


@dataclass
class UpdateTwoDSnapshotResponse(TcBaseObj):
    """
    Return structure for update 2DSnapshot operation
    
    :var updateSnapshotInfo: A map containing client ids and output structure as key/value pairs.
    :var serviceData: The Service Data containing the error information.
    """
    updateSnapshotInfo: UpdateTwoDSnapshotOutputMap = None
    serviceData: ServiceData = None


@dataclass
class UserAgentDataInfo(TcBaseObj):
    """
    This structure holds the information about the client application that initiated the launch.
    
    :var userApplication: An optional parameter referencing the client who initiates the launch.
    :var userAppVersion: An optional parameter referencing the version of the client that initiated launch.
    :var userAdditionalInfo: An optional parameter referencing the additional information of client application in form
    of key/value pair (if any).
    """
    userApplication: str = ''
    userAppVersion: str = ''
    userAdditionalInfo: KeyValueMap = None


@dataclass
class VVITicketsResponse(TcBaseObj):
    """
    Used to return information from the 'createLaunchFile' operation. The structure holds the 'serviceData' object and
    a FMS transient read ticket corresponding to the launch file (VVI or VFZ).
    
    :var ticket: The FMS transient read ticket of the launch file (VVI or VFZ) generated for the objects that can be
    launched. The file will be placed in the transient file volume and the caller will need to download it from there
    with the ticket sent by the service.
    :var serviceData: SOA Framework object containing error information. In cases where objects cannot be launched,
    error message, codes will be mapped to respective object id in the list of partial errors.
    """
    ticket: str = ''
    serviceData: ServiceData = None


@dataclass
class CreateTwoDSnapshotOutput(TcBaseObj):
    """
    Output structure for create 2D Snapshot operation
    
    :var dataset: Dataset object that was created.
    :var form: The form attached to the snapshot dataset.
    :var references: List of named references associated with dataset.
    :var attachRelation: The datasets relation VisItemRevCreatedSnapshot2D that attached the 2D snapshot to to the 
    parent ItemRevision object.
    """
    dataset: Dataset = None
    form: BusinessObject = None
    references: List[NamedRefsInDataset] = ()
    attachRelation: ImanRelation = None


@dataclass
class CreateTwoDSnapshotResponse(TcBaseObj):
    """
    Return structure for create 2DSnapshot operation
    
    :var snapshotOutput: A map containing client ids and output structure as key/value pairs.
    :var serviceData: The Service Data containing the error information.
    """
    snapshotOutput: CreateTwoDSnapshotOutputMap = None
    serviceData: ServiceData = None


@dataclass
class DatasetInfo(TcBaseObj):
    """
    This structure holds the information related to creation of the markup Dataset.
    
    :var tool: (Required) The name of the tool used to open the created markup Dataset.
    :var datasetType: (Required) The Dataset type of the markup Dataset to be created.
    :var datasetName: (Required) The name provided by the client application to the markup Dataset.
    :var datasetDesc: (Optional) The description provided by the client application to be associated to the markup
    Dataset.
    """
    tool: str = ''
    datasetType: str = ''
    datasetName: str = ''
    datasetDesc: str = ''


"""
CreateTwoDSnapshotOutputMap
"""
CreateTwoDSnapshotOutputMap = Dict[str, CreateTwoDSnapshotOutput]


"""
KeyValueMap
"""
KeyValueMap = Dict[str, str]


"""
MarkupOutputMap
"""
MarkupOutputMap = Dict[str, MarkupOutput]


"""
MarkupUpdateOutputMap
"""
MarkupUpdateOutputMap = Dict[str, MarkupUpdateOutput]


"""
SessionModelOutputMap
"""
SessionModelOutputMap = Dict[str, SessionModelOutput]


"""
SessionModelUpdateOutputMap
"""
SessionModelUpdateOutputMap = Dict[str, SessionModelUpdateOutput]


"""
SessionToSessionModelMap
"""
SessionToSessionModelMap = Dict[str, SaveSessionOutput]


"""
StringArrayMap
"""
StringArrayMap = Dict[str, List[str]]


"""
UpdateTwoDSnapshotOutputMap
"""
UpdateTwoDSnapshotOutputMap = Dict[str, UpdateTwoDSnapshotOutput]



from __future__ import annotations

from tcsoa.gen.StructureManagement._2012_02.StructureVerification import CompareNetEffectivityResponse, EquivalentSetElement, GetPropertyComparisonDetailsResponse, GetDescendentComparisonDetailsResponse, PropagationResponse, GetPartitionComparisonDetailsResponse, EquivalentLines, ACInput, GetPredecessorComparisonDetailsResponse, AsyncACInput, PropagationInput, BatchDetails, AsyncDetails, GetAssignmentComparisonDetailsResponse
from tcsoa.gen.StructureManagement._2010_09.StructureVerification import AccountabilityCheckResponse
from tcsoa.gen.StructureManagement._2012_02.IncrementalChange import BomLineInfo
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcService


class StructureVerificationService(TcService):

    @classmethod
    def getPropertyComparisonDetails(cls, equivalentObjects: List[EquivalentSetElement]) -> GetPropertyComparisonDetailsResponse:
        """
        method to retrieve details for supplied properties on the provided equivalent sets of lines.
        """
        return cls.execute_soa_method(
            method_name='getPropertyComparisonDetails',
            library='StructureManagement',
            service_date='2012_02',
            service_name='StructureVerification',
            params={'equivalentObjects': equivalentObjects},
            response_cls=GetPropertyComparisonDetailsResponse,
        )

    @classmethod
    def compareNetEffectivity(cls, lines: List[EquivalentLines], ignoreOverlapErrors: bool, useStructureConfiguration: bool, returnOnFirstMismatch: bool) -> CompareNetEffectivityResponse:
        """
        effectivity of 2 sets of lines that are deemed equivalent in some form.
        """
        return cls.execute_soa_method(
            method_name='compareNetEffectivity',
            library='StructureManagement',
            service_date='2012_02',
            service_name='StructureVerification',
            params={'lines': lines, 'ignoreOverlapErrors': ignoreOverlapErrors, 'useStructureConfiguration': useStructureConfiguration, 'returnOnFirstMismatch': returnOnFirstMismatch},
            response_cls=CompareNetEffectivityResponse,
        )

    @classmethod
    def propagateProperties(cls, input: List[PropagationInput]) -> PropagationResponse:
        """
        method to propagate properties.
        """
        return cls.execute_soa_method(
            method_name='propagateProperties',
            library='StructureManagement',
            service_date='2012_02',
            service_name='StructureVerification',
            params={'input': input},
            response_cls=PropagationResponse,
        )

    @classmethod
    def accountabilityCheck(cls, input: ACInput, batchDetails: BatchDetails) -> AccountabilityCheckResponse:
        """
        The operation will call the existing accountability check functions, which will generate a check result for
        report in the colored display.
        """
        return cls.execute_soa_method(
            method_name='accountabilityCheck',
            library='StructureManagement',
            service_date='2012_02',
            service_name='StructureVerification',
            params={'input': input, 'batchDetails': batchDetails},
            response_cls=AccountabilityCheckResponse,
        )

    @classmethod
    def accountabilityCheckAsync(cls, acinput: AsyncACInput, asyncDetails: AsyncDetails) -> None:
        """
        asynchronous implementation for the accountabilityCheck.
        """
        return cls.execute_soa_method(
            method_name='accountabilityCheckAsync',
            library='StructureManagement',
            service_date='2012_02',
            service_name='StructureVerification',
            params={'acinput': acinput, 'asyncDetails': asyncDetails},
            response_cls=None,
        )

    @classmethod
    def getAssignmentComparisonDetails(cls, equivalentObjects: List[EquivalentLines], partRelationTypes: List[str], toolRelationTypes: List[str], compareLA: int, compareManual: bool) -> GetAssignmentComparisonDetailsResponse:
        """
        get the details of any differences between assignments for the supplied src and target objects.
        """
        return cls.execute_soa_method(
            method_name='getAssignmentComparisonDetails',
            library='StructureManagement',
            service_date='2012_02',
            service_name='StructureVerification',
            params={'equivalentObjects': equivalentObjects, 'partRelationTypes': partRelationTypes, 'toolRelationTypes': toolRelationTypes, 'compareLA': compareLA, 'compareManual': compareManual},
            response_cls=GetAssignmentComparisonDetailsResponse,
        )

    @classmethod
    def getDescendentComparisonDetails(cls, input: List[EquivalentLines], ignoreStructureSpecific: bool) -> GetDescendentComparisonDetailsResponse:
        """
        for each input equivalent set capture the response of getDescendentComparisonDetails method. Has the list of
        details for each input set and serviceData to capture partial errors.
        """
        return cls.execute_soa_method(
            method_name='getDescendentComparisonDetails',
            library='StructureManagement',
            service_date='2012_02',
            service_name='StructureVerification',
            params={'input': input, 'ignoreStructureSpecific': ignoreStructureSpecific},
            response_cls=GetDescendentComparisonDetailsResponse,
        )

    @classmethod
    def getPartitionComparisonDetails(cls, input: List[EquivalentLines]) -> GetPartitionComparisonDetailsResponse:
        """
        service to get details on a mismatch of parititions.
        """
        return cls.execute_soa_method(
            method_name='getPartitionComparisonDetails',
            library='StructureManagement',
            service_date='2012_02',
            service_name='StructureVerification',
            params={'input': input},
            response_cls=GetPartitionComparisonDetailsResponse,
        )

    @classmethod
    def getPredecessorComparisonDetails(cls, input: List[EquivalentLines], ignoreStructureSpecific: bool) -> GetPredecessorComparisonDetailsResponse:
        """
        for a given equivalent set of lines/object, compute and return the list of predecessor detail elements and flag
        if different.
        """
        return cls.execute_soa_method(
            method_name='getPredecessorComparisonDetails',
            library='StructureManagement',
            service_date='2012_02',
            service_name='StructureVerification',
            params={'input': input, 'ignoreStructureSpecific': ignoreStructureSpecific},
            response_cls=GetPredecessorComparisonDetailsResponse,
        )


class IncrementalChangeService(TcService):

    @classmethod
    def carryOver(cls, bomLineInfos: List[BomLineInfo]) -> ServiceData:
        """
        This operation carries the IncrementalChangeElement (ICE) object(s) from the source line to the corresponding
        target line by cloning them. 
        There are two BMIDE constants that were introduced for this functionality
          
        Type Constant Fnd0EnableIceCarryOver
        Property Constant Fnd0AttrIcesToExclude
        
        - New IncrementalChangeElement object(s) are attached to the original incremental change revision.
        
        - Add changes on the occurrence are ignored - It does not make sense to have two occurrences appearing at the
        same time at different locations.
        
        - GDELine objects are not supported.
        
        - For attribute changes the context of the absolute occurrence data is the new parent BOMView Revision.
        
        - If there is no write access on incremental change revision, the carryover is allowed with a programmatic
        bypass.
        - If there is any failure while cloning any IncrementalChangeElement object that is being carried over, the
        entire operation will rollback.
        
        - Carryover is not possible if we have the following two conditions:
        - Both source and target BomView Revision item type are not available in the BMIDE Type Constant
        Fnd0EnableIceCarryOver.
        - BOM-BOP case - If the source line is a BOMLine and the target line is a BOP (Bill of Process) line or vice
        versa
        
        
        
        Use cases:
        User wants to carry over IncrementalChangeElement object(s) when doing a Copy-Paste or a Cut-Paste of a BOMLine
        having IncrementalChangeElement object(s), from one structure partition to another partition within the Plant
        BOP.
        User wants to carry over IncrementalChangeElement object(s) when doing a Copy-Paste or a Cut-Paste of a BOMLine
        having IncrementalChangeElement object(s), from Product BOP and to another partition in the Plant BOP.
        
        """
        return cls.execute_soa_method(
            method_name='carryOver',
            library='StructureManagement',
            service_date='2012_02',
            service_name='IncrementalChange',
            params={'bomLineInfos': bomLineInfos},
            response_cls=ServiceData,
        )



from __future__ import annotations

from tcsoa.gen.BusinessObjects import RequestObject, AppInterface
from tcsoa.gen.Ai._2006_03.Ai import CommitFilesResponse, Configuration, GenerateStructureResponse, GenerateFullSyncRequestResponse, GetStructureReadTicketResponse, GetNextApprovedRequestResponse, RequestDetail, GetPropertiesOfObjectsResponse, CommitFileData, StateFilter, ProjectFilter, ObjectsExistResponse, GetStructureWriteTicketResponse, FileType, CommitStructureFileResponse, ProjectCreationData, GetFileWriteTicketsResponse, UpdateType, GetFileReadTicketsResponse, ProjectInfo, ApplicationRef, CreatePublishRequestResponse, StatusInfo, FileTicket
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcService


class AiService(TcService):

    @classmethod
    def getProjectsInfo(cls, aiIds: List[AppInterface]) -> ServiceData:
        """
        Get details of the specified AI Objects.
        
        Exceptions:
        >Thrown on service failure.
        """
        return cls.execute_soa_method(
            method_name='getProjectsInfo',
            library='Ai',
            service_date='2006_03',
            service_name='Ai',
            params={'aiIds': aiIds},
            response_cls=ServiceData,
        )

    @classmethod
    def getPropertiesOfObjects(cls, appRefs: List[ApplicationRef], propertySetNames: List[str]) -> GetPropertiesOfObjectsResponse:
        """
        Get Properties of Teamcenter obects based on a propertyset created using the plmxml admin application.
        
        Exceptions:
        >Thrown on service failure.
        """
        return cls.execute_soa_method(
            method_name='getPropertiesOfObjects',
            library='Ai',
            service_date='2006_03',
            service_name='Ai',
            params={'appRefs': appRefs, 'propertySetNames': propertySetNames},
            response_cls=GetPropertiesOfObjectsResponse,
        )

    @classmethod
    def getRequestsForProject(cls, projectId: AppInterface, filter: StateFilter) -> ServiceData:
        """
        Given a Project(AI) object, get the RequestObjects based on the optional filter.
        default filter used: RequestType Sync and RequestState - pending.
        
        Exceptions:
        >Thrown on service failure.
        """
        return cls.execute_soa_method(
            method_name='getRequestsForProject',
            library='Ai',
            service_date='2006_03',
            service_name='Ai',
            params={'projectId': projectId, 'filter': filter},
            response_cls=ServiceData,
        )

    @classmethod
    def getRequestsInfo(cls, reqIds: List[RequestObject]) -> ServiceData:
        """
        Given a vector of RequestObjects, get details on those. This can be used if the
        RequestObject Ids are known. If not, GetProjectRequests can be used.
        
        Exceptions:
        >Thrown on service failure.
        """
        return cls.execute_soa_method(
            method_name='getRequestsInfo',
            library='Ai',
            service_date='2006_03',
            service_name='Ai',
            params={'reqIds': reqIds},
            response_cls=ServiceData,
        )

    @classmethod
    def getStructureReadTicket(cls, id: RequestObject, type: UpdateType) -> GetStructureReadTicketResponse:
        """
        Used to download the ticket for the plmxml file associated with the RequestObject.
        This ticket is then to be used with FCC client Proxy to download the file.
        
        Exceptions:
        >Thrown on service failure.
        """
        return cls.execute_soa_method(
            method_name='getStructureReadTicket',
            library='Ai',
            service_date='2006_03',
            service_name='Ai',
            params={'id': id, 'type': type},
            response_cls=GetStructureReadTicketResponse,
        )

    @classmethod
    def getStructureWriteTicket(cls, originalFileName: str) -> GetStructureWriteTicketResponse:
        """
        Used to download the ticket for the plmxml file that will be uploaded by the client. This ticket
        is then to be used with FCC client Proxy to actually upload the file.
        
        Exceptions:
        >Thrown on service failure.
        """
        return cls.execute_soa_method(
            method_name='getStructureWriteTicket',
            library='Ai',
            service_date='2006_03',
            service_name='Ai',
            params={'originalFileName': originalFileName},
            response_cls=GetStructureWriteTicketResponse,
        )

    @classmethod
    def commitFiles(cls, reqObj: RequestObject, data: List[CommitFileData]) -> CommitFilesResponse:
        """
        After a call to getFileWriteTickets - this method is to be used to create TeamCenter file objects - referencing
        the files in the volume.
        
        Exceptions:
        >Thrown on service failure.
        """
        return cls.execute_soa_method(
            method_name='commitFiles',
            library='Ai',
            service_date='2006_03',
            service_name='Ai',
            params={'reqObj': reqObj, 'data': data},
            response_cls=CommitFilesResponse,
        )

    @classmethod
    def commitStructureFile(cls, id: RequestObject, ticket: FileTicket, pUpdate: UpdateType) -> CommitStructureFileResponse:
        """
        This method is to be used to save a plmxml file after getting the write ticket using the
        GetStructureWriteTicket method.
        
        Exceptions:
        >Thrown on service failure.
        """
        return cls.execute_soa_method(
            method_name='commitStructureFile',
            library='Ai',
            service_date='2006_03',
            service_name='Ai',
            params={'id': id, 'ticket': ticket, 'pUpdate': pUpdate},
            response_cls=CommitStructureFileResponse,
        )

    @classmethod
    def objectsExist(cls, objIds: List[ApplicationRef]) -> ObjectsExistResponse:
        """
        Given 1 or more ApplicationRef objects - find the corresponding TeamCenter Object(s). Typically,
        the ApplicationRef is obtained from a plmxml file. The ApplicationRef for Teamcenter objects is
        (from a client point of view) - application='TcEng', label=teamcenter_uid, version=teamcenter_uid.
        
        Exceptions:
        >Thrown on service failure.
        """
        return cls.execute_soa_method(
            method_name='objectsExist',
            library='Ai',
            service_date='2006_03',
            service_name='Ai',
            params={'objIds': objIds},
            response_cls=ObjectsExistResponse,
        )

    @classmethod
    def processPublishRequest(cls, id: RequestObject) -> ServiceData:
        """
        Process(import) the plmxml corresponding to the RequestObject
        
        Exceptions:
        >Thrown on service failure.
        """
        return cls.execute_soa_method(
            method_name='processPublishRequest',
            library='Ai',
            service_date='2006_03',
            service_name='Ai',
            params={'id': id},
            response_cls=ServiceData,
        )

    @classmethod
    def processStructure(cls, transferModeName: str, plmxmlFileId: str, config: Configuration) -> ServiceData:
        """
        Import a plmxml for a previously uploaded plmxml via getStructureWriteTicket, fcc method to upload the file and
        commitStructureFile (with no associated RequestObject - non AI plmxml import)
        
        Note: 
        For importing plmxml that contains datasets or other attachments you need to use 'importObjectsFromPLMXML' from
        'globalmultisite.ImportExportService'.
        
        Exceptions:
        >Thrown on service failure.
        """
        return cls.execute_soa_method(
            method_name='processStructure',
            library='Ai',
            service_date='2006_03',
            service_name='Ai',
            params={'transferModeName': transferModeName, 'plmxmlFileId': plmxmlFileId, 'config': config},
            response_cls=ServiceData,
        )

    @classmethod
    def setExchangeMessage(cls, id: RequestObject, stateMsg: str, info: StatusInfo) -> ServiceData:
        """
        Set any StatusInfo on the RequestObject and state description
        
        Exceptions:
        >Thrown on service failure.
        """
        return cls.execute_soa_method(
            method_name='setExchangeMessage',
            library='Ai',
            service_date='2006_03',
            service_name='Ai',
            params={'id': id, 'stateMsg': stateMsg, 'info': info},
            response_cls=ServiceData,
        )

    @classmethod
    def setProjectsInfo(cls, projectIds: List[AppInterface], infos: List[ProjectInfo]) -> ServiceData:
        """
        Set details of the specified AI Objects from the supplied vector of ProjectInfo objects. You can only set
        targetAppProjectId, description and name fields.
        
        Exceptions:
        >Thrown on service failure.
        """
        return cls.execute_soa_method(
            method_name='setProjectsInfo',
            library='Ai',
            service_date='2006_03',
            service_name='Ai',
            params={'projectIds': projectIds, 'infos': infos},
            response_cls=ServiceData,
        )

    @classmethod
    def startExchange(cls, id: RequestObject) -> ServiceData:
        """
        Set Communicating State on the RequestObject. To be called before uploading or downloading
        files to the RequestObject
        
        Exceptions:
        >Thrown on service failure.
        """
        return cls.execute_soa_method(
            method_name='startExchange',
            library='Ai',
            service_date='2006_03',
            service_name='Ai',
            params={'id': id},
            response_cls=ServiceData,
        )

    @classmethod
    def createProjects(cls, projectDatas: List[ProjectCreationData]) -> ServiceData:
        """
        Create ApplicationInterface Objects in Teamcenter based on the input values
        
        Exceptions:
        >Thrown on service failure.
        """
        return cls.execute_soa_method(
            method_name='createProjects',
            library='Ai',
            service_date='2006_03',
            service_name='Ai',
            params={'projectDatas': projectDatas},
            response_cls=ServiceData,
        )

    @classmethod
    def createPublishRequest(cls, id: AppInterface, detail: RequestDetail, plmxmlFileName: str) -> CreatePublishRequestResponse:
        """
        Create a new RequestObject of type Publish on the specified AI Object.
        
        Exceptions:
        >Thrown on service failure.
        """
        return cls.execute_soa_method(
            method_name='createPublishRequest',
            library='Ai',
            service_date='2006_03',
            service_name='Ai',
            params={'id': id, 'detail': detail, 'plmxmlFileName': plmxmlFileName},
            response_cls=CreatePublishRequestResponse,
        )

    @classmethod
    def deleteProjects(cls, projectIds: List[AppInterface]) -> ServiceData:
        """
        Delete the specified AI Objects from the Database.
        
        Exceptions:
        >Thrown on service failure.
        """
        return cls.execute_soa_method(
            method_name='deleteProjects',
            library='Ai',
            service_date='2006_03',
            service_name='Ai',
            params={'projectIds': projectIds},
            response_cls=ServiceData,
        )

    @classmethod
    def deleteRequests(cls, requestIds: List[RequestObject]) -> ServiceData:
        """
        Given 1 or more requestObjects - delete them from the database
        
        Exceptions:
        >Thrown on service failure.
        """
        return cls.execute_soa_method(
            method_name='deleteRequests',
            library='Ai',
            service_date='2006_03',
            service_name='Ai',
            params={'requestIds': requestIds},
            response_cls=ServiceData,
        )

    @classmethod
    def endExchange(cls, id: RequestObject, stateMsg: str, info: StatusInfo) -> ServiceData:
        """
        Set Completed state on the Sync RequestObject or Pending state on a Publish RequestObject.
        StatusInfo and state description can be set by client
        
        Exceptions:
        >Thrown on service failure.
        """
        return cls.execute_soa_method(
            method_name='endExchange',
            library='Ai',
            service_date='2006_03',
            service_name='Ai',
            params={'id': id, 'stateMsg': stateMsg, 'info': info},
            response_cls=ServiceData,
        )

    @classmethod
    def generateFullSyncRequest(cls, id: AppInterface, name: str, description: str, baseRefs: List[ApplicationRef]) -> GenerateFullSyncRequestResponse:
        """
        Generate a Sync Request on a new AI or Existing AI object.
        
        Exceptions:
        >Thrown on service failure.
        """
        return cls.execute_soa_method(
            method_name='generateFullSyncRequest',
            library='Ai',
            service_date='2006_03',
            service_name='Ai',
            params={'id': id, 'name': name, 'description': description, 'baseRefs': baseRefs},
            response_cls=GenerateFullSyncRequestResponse,
        )

    @classmethod
    def generateStructure(cls, idsToProcess: List[ApplicationRef], transferModeName: str, config: Configuration, serverMode: int) -> GenerateStructureResponse:
        """
        Generate a plmxml for the given set of ids.
        
        Exceptions:
        >Thrown on service failure.
        """
        return cls.execute_soa_method(
            method_name='generateStructure',
            library='Ai',
            service_date='2006_03',
            service_name='Ai',
            params={'idsToProcess': idsToProcess, 'transferModeName': transferModeName, 'config': config, 'serverMode': serverMode},
            response_cls=GenerateStructureResponse,
        )

    @classmethod
    def getFileReadTickets(cls, fileIds: List[ApplicationRef]) -> GetFileReadTicketsResponse:
        """
        Used to download the tickets for the files referenced by the plmxml file previously exported from Teamcenter.
        These tickets are then to be used with FCC client Proxy to actually download the files.
        
        Exceptions:
        >Thrown on service failure.
        """
        return cls.execute_soa_method(
            method_name='getFileReadTickets',
            library='Ai',
            service_date='2006_03',
            service_name='Ai',
            params={'fileIds': fileIds},
            response_cls=GetFileReadTicketsResponse,
        )

    @classmethod
    def getFileWriteTickets(cls, originalFileNames: List[str], fileTypes: List[FileType]) -> GetFileWriteTicketsResponse:
        """
        Used to download the tickets for the files referenced by the plmxml file created by a client application.
        These tickets are then to be used with FCC client Proxy to actually upload(commit) the files.
        
        Exceptions:
        >Thrown on service failure.
        """
        return cls.execute_soa_method(
            method_name='getFileWriteTickets',
            library='Ai',
            service_date='2006_03',
            service_name='Ai',
            params={'originalFileNames': originalFileNames, 'fileTypes': fileTypes},
            response_cls=GetFileWriteTicketsResponse,
        )

    @classmethod
    def getNextApprovedRequest(cls, projectId: AppInterface, curReq: RequestObject) -> GetNextApprovedRequestResponse:
        """
        Given a  RequestObject get the next approved pending sync RequestObject
        
        Exceptions:
        >Thrown on service failure.
        """
        return cls.execute_soa_method(
            method_name='getNextApprovedRequest',
            library='Ai',
            service_date='2006_03',
            service_name='Ai',
            params={'projectId': projectId, 'curReq': curReq},
            response_cls=GetNextApprovedRequestResponse,
        )

    @classmethod
    def getProjects(cls, filter: ProjectFilter) -> ServiceData:
        """
        Get a list of projects that are available on this database based on the filter specified.
        Suggest atleast filtering based on type of AppInterface
        
        Exceptions:
        >Thrown on service failure.
        """
        return cls.execute_soa_method(
            method_name='getProjects',
            library='Ai',
            service_date='2006_03',
            service_name='Ai',
            params={'filter': filter},
            response_cls=ServiceData,
        )



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject
from tcsoa.gen.Cad._2020_01.AppSessionManagement import CreateOrUpdateSessionResponse, OpenSavedSessionFilter, OpenSessionResponse, SessionInfo
from typing import List
from tcsoa.base import TcService


class AppSessionManagementService(TcService):

    @classmethod
    def openSavedSession(cls, sessionsToOpen: List[BusinessObject], filter: OpenSavedSessionFilter) -> OpenSessionResponse:
        """
        This operation is used to open the documents and product structures that were saved to a session object using
        'createOrUpdateSavedSession'.  For product structures, the configuration rules persisted with the session are
        used to recreate the correct BOM system configuration in effect when the session was last authored.  The client
        can either open all the documents and product structures associated to the session or selectively open by
        providing 'OpenSavedSessionFilter'. The session object type that can be opened by this operation is
        Fnd0AppSession.
        
        Use cases:
        The user may have opened multiple product structures, applied search filters on them and along with  it  may
        have opened other objects like Dataset or Form. The user then issues a save session command to capture the
        state of the session so that it can be restored for continuation of work or share it other users with review
        markups. 
        
        The mechanism for a client application(CAD) which captures the CAD related session setting in a session file
        and saves the opened products, documents and its state in Teamcenter as session, contains steps as below.
        
        1.    User opens product structures, Dataset, creates a review markup and saves the session.
        2.    Client application creates a Dataset that will contain the session assets information like Markup,
        Snapshot etc. using 'createDatasets'. The client application then invokes the 'createOrUpdateSavedSession'
        where the input are the objects that were opened in the session and Dataset containing session assets. This
        step creates the session object and associates the input objects to the session with relevant relations as per
        the data model. Associated Dataset and ImanFile objects are then accessible to other authorized clients through
        the PLM system.
        3.    Another user could search for the session created in step 2 and issue a command in the application to
        open the session. The client application now invokes this operation to get all the objects and product
        structures that were associated to the session and present it to the user the way he persisted those in
        Teamcenter. The stable Id that uniquely identifies the association of a product structure or object to the
        session is also returned so that it can used to update the session when the user modifies the session.
        """
        return cls.execute_soa_method(
            method_name='openSavedSession',
            library='Cad',
            service_date='2020_01',
            service_name='AppSessionManagement',
            params={'sessionsToOpen': sessionsToOpen, 'filter': filter},
            response_cls=OpenSessionResponse,
        )

    @classmethod
    def createOrUpdateSavedSession(cls, sessionsToCreateOrUpdate: List[SessionInfo]) -> CreateOrUpdateSessionResponse:
        """
        This operation creates or updates a session data model that captures product structure configuration rules, non
        structure object references (e.g. 2D drawings), and application specific data (e.g. markups, snaphsots) from a
        visualization enabled application. The session model is a Fnd0AppSession(Session) Object referencing objects
        opened in client applications. The operation returns unique, copy stable ids for all the objects referenced by
        the session for persistence of it into CAD session files. Client specific objects can be associated to
        Fnd0AppSession with GRM relations or as sub class Fnd0ProductSessionData where the fnd0OwningObject property is
        referencing Fnd0AppSession.
        
        Use cases:
        The user may have opened multiple product structures, applied search filters on them and along with  it  may
        have opened other objects like Dataset or Form. The user then issues a save session command to capture the
        state of the session so that it can be restored for continuation of work or share it other users with review
        markups. 
        
        The mechanism for a client application(CAD) which captures the CAD related session setting in a session file
        and saves the opened products, documents and its state in Teamcenter as session, contains steps as below.
        
        User creates a Session
        - User opens product structures, Dataset, creates a review markup and saves the session.
        - Client application creates a Dataset that will contain the session assets information like Markup, Snapshot
        etc. using 'createDatasets' or using 'BaseObjectAttachInfo.baseObjectToCreateOrUpdate'. 
        - The client application then invokes the 'createOrUpdateSavedSession' where the input are the objects that
        were opened in the session and Dataset containing session assets. This step creates the session object and
        associates the input objects to the session with relevant relations as per the data model. Associated Dataset
        and ImanFile objects are then accessible to other authorized clients through the PLM system.
        
        
        
        User updates a Session
        - User searches for the Fnd0AppSession he has saved.
        - Opens it in the client application that supports opening of Fnd0AppSession.
        - The client application invokes 'openSavedSession' operation which returns the objects associted to the
        session with its corresponding stable Ids.
        - The application restores his session as persisted by the Fnd0AppSession.
        - User now closes some documents, opens new documents and/or modifes existing objects.
        - User now issues a command to save the session
        - The application invokes 'createOrUpdateSavedSession' operation to update the object association to the
        session by identifying the association via stable Id. 
        
        
        
        Note: As a best practice, if the configuration or search recipe of a product structure in an existing session
        is changed and saved by the user it is recommended that the application removes the the product structure from
        the session and adds the changed product structure to the session with the same stableId or using a new stable
        id.
        """
        return cls.execute_soa_method(
            method_name='createOrUpdateSavedSession',
            library='Cad',
            service_date='2020_01',
            service_name='AppSessionManagement',
            params={'sessionsToCreateOrUpdate': sessionsToCreateOrUpdate},
            response_cls=CreateOrUpdateSessionResponse,
        )



from __future__ import annotations

from tcsoa.gen.BusinessObjects import TransferMode, Fnd0DiagramTmplRevision
from typing import Dict, List
from tcsoa.base import TcBaseObj
from dataclasses import dataclass


@dataclass
class CreateOrUpdateTemplateInputInfo1(TcBaseObj):
    """
    CreateOrUpdateTemplateInputInfo1 structure represents the parameters required to create a diagram template.
    
    :var available: If true, the Diagram Template is available for use.
    :var tmplStencilFileTickets: A list of FMS tickets to the Diagram tool specific stencils or Template files.
    :var tmplMappingFileTicket: FMS ticket to the Property Map xml file.
    :var membershipRule: The Transfer mode, which will be used for traversing the structure for the diagram root object.
    :var relationRule: The Relation Rule which is the list of relations between the objects shown on the diagram.
    :var diagramTmplRev: The updated template object.
    :var propNamevsPropValueMap: A map of property names and values (string/string). Valid keys are Description, Name,
    templateName and ID.
    :var hidePorts: If true, the interface shapes will not be shown on the created diagram.
    """
    available: bool = False
    tmplStencilFileTickets: List[str] = ()
    tmplMappingFileTicket: str = ''
    membershipRule: TransferMode = None
    relationRule: List[str] = ()
    diagramTmplRev: Fnd0DiagramTmplRevision = None
    propNamevsPropValueMap: DiagramTemplatePropNamevsPropValueMap = None
    hidePorts: bool = False


"""
This  map contains the property name and its value for a Diagram Template
"""
DiagramTemplatePropNamevsPropValueMap = Dict[str, str]



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcBaseObj
from dataclasses import dataclass
from typing import Dict, List


@dataclass
class TraversedSecondaryObjectInfo(TcBaseObj):
    """
    Information about traversal information of the secondary object.
    
    :var secondaryObjectType: The business object name of the secondary object that is traversed from primary object.
    It can be any business object name.
    :var imanRelationType: The ImanRelation type name that relates the primary and secondary object. It can be any iman
    relation.
    :var imanRelationDescription: Description of the Iman Relation.
    :var isSecondaryIncluded: If true, the secondary object type is included in traversal.
    :var isSecondaryEditable: If true,the user has option to modify the traversal of secondary object.
    :var isPrimaryIncluded: If true, the primary object type is included in traversal.
    :var isPrimaryEditable: If true,the user has option to modify the traversal of primary object.
    """
    secondaryObjectType: str = ''
    imanRelationType: str = ''
    imanRelationDescription: str = ''
    isSecondaryIncluded: bool = False
    isSecondaryEditable: bool = False
    isPrimaryIncluded: bool = False
    isPrimaryEditable: bool = False


@dataclass
class ClosureRuleTraversalInput(TcBaseObj):
    """
    The structure related to selected closure rule and the bomline by the user.
    
    :var topBOMLine: The top BOMLine for traversal information.
    :var closureRuleName: The closure rule name for which the clauses have to be updated.
    """
    topBOMLine: BusinessObject = None
    closureRuleName: str = ''


@dataclass
class ClosureRuleTraversalResponse(TcBaseObj):
    """
    The closure rule traversal response.
    
    :var traversalInfo: A map (string,TraversedSecondaryObjectInfo) of primary object type name and secondary object
    traversal information.
    :var serviceData: Service data containing partial errors.
    """
    traversalInfo: PrimaryToSecondaryTraversalMap = None
    serviceData: ServiceData = None


"""
The map of the primary object type and a structure holding the information about its secondary object types.
"""
PrimaryToSecondaryTraversalMap = Dict[str, List[TraversedSecondaryObjectInfo]]



from __future__ import annotations

from tcsoa.gen.BusinessObjects import ClosureRule, WorkspaceObject, Dataset, User, ValidationData, ValidationResult, ItemRevision
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcBaseObj
from dataclasses import dataclass
from datetime import datetime


@dataclass
class ExternalRuleInfo(TcBaseObj):
    """
    The ExternalRuleInfo struct represents the information for external rule dataset for validation data.
    
    :var useExistingObject: A bool value that indicates if an existing closure rule object will be used.
    :var extenralRuleObject: Pointer to an existing closure rule object to be used.
    :var ruleDatasetInfo: The structure contains necessary data for creating a rule dataset.
    """
    useExistingObject: bool = False
    extenralRuleObject: Dataset = None
    ruleDatasetInfo: DatasetInfo = None


@dataclass
class GetValResultsInput(TcBaseObj):
    """
    The GetValResultsInput struct  holds the input data for getValidationResult .  There is no one to one mapping
    between these two input lists. When both lists are NULL, all validation results in the database will be returned.
    
    :var targetObjects: A list of  target objects. It can be NULL, when it is NULL, it means validation result with any
    target object is ok.
    :var validationDatas: A list of  ValidationData objects. It can be NULL, when it is NULL; it means validation
    result with any validation data is ok.
    """
    targetObjects: List[WorkspaceObject] = ()
    validationDatas: List[ValidationData] = ()


@dataclass
class GetValidationResultResponse(TcBaseObj):
    """
    Holds the response for getValidationResult.
    
    :var results: A vector of ValidationResult objects.
    :var serviceData: Teamcenter::Soa::Server::ServiceData is the Framework class that holds model objects and partial
    errors.
    """
    results: List[ValidationResult] = ()
    serviceData: ServiceData = None


@dataclass
class AttributeInfo(TcBaseObj):
    """
    The AttributeInfo struct represents the parameter <name, value, operation> tuple for both ValidationData and
    ValidationResult.
    
    :var name: A parameter name string.
    :var value: A parameter value string.
    :var operation: A string indicating the comparison method, valid values are = or empty string.
    """
    name: str = ''
    value: str = ''
    operation: str = ''


@dataclass
class ReportFileInfo(TcBaseObj):
    """
    The structure contains the information for new report datasets that are associated with ValidationResult objects.
    
    :var useExistingObject: A bool value that indicates if an existing dataset will be updated.
    :var reportFileObject: Pointer to a valid ImanFile object for the dataset.
    :var reportDatasetInfo: The name reference data needed for updating the given file object.
    """
    useExistingObject: bool = False
    reportFileObject: Dataset = None
    reportDatasetInfo: DatasetInfo = None


@dataclass
class ValidationAgentInfo(TcBaseObj):
    """
    The ValidationAgentInfo structure is the main input to the createValidationData and updateValidationData service.
    This struct refers to the ValidationData, closure rule, external rule and requirement Used to create or update
    those objects.
    
    :var existingValidationAgent: The object to be updated. When it is NULL, a new ValidationAgent object will be
    created.
    :var clientId: Client supplied identifier string that helps the client map from the input for the object(s)
    created. It should be unique in the client session.
    :var name: The name for a validation agent.
    :var description: The description for a validation agent.
    :var utilityCommand: The validation utility command.
    :var utilityArgs: The arguments for the validation utility command.
    :var closureRule: Struct ClosureRuleInfo describing the closure rule.
    :var checkUserPriviledge: When it is set to true, only system administrators or validation administrators can
    create or modify a ValidationAgent When it is set to false, it will bypass the privilege checking.
    """
    existingValidationAgent: ValidationData = None
    clientId: str = ''
    name: str = ''
    description: str = ''
    utilityCommand: str = ''
    utilityArgs: str = ''
    closureRule: ClosureRuleInfo = None
    checkUserPriviledge: bool = False


@dataclass
class ValidationAgentOutput(TcBaseObj):
    """
    The ValidationAgentOutput struct holds the information for created or updated validation agent and it child
    validation data.
    
    :var clientId: Unique client identifier string.
    :var validationAgent: Pointer to a validation agent.
    :var childValidationData: The associated validation checker objects for the validation agent.
    """
    clientId: str = ''
    validationAgent: ValidationData = None
    childValidationData: List[ValidationData] = ()


@dataclass
class ValidationDataInfo(TcBaseObj):
    """
    The ValidationDataInfo struct is the main input to the createOrUpdateValidationData service. This struct refers to
    the ValidationData, closure rule, external rule and requirement used to create or update those objects.
    
    :var existingValidationData: The object to be updated. When it is NULL or points to a NULL TAG, a new
    ValidationResult will be created
    :var clientId: Client supplied identifier string that helps the client map from the input for the create to the
    object(s) created. This client ID should be unique in the client session. It is the clients way of identifying the
    ValidationData (not necessarily an attribute persisted in Teamcenter).
    :var checkUserPriviledge: When it is set to true, only SA or GA can create or modify a ValidationData object. When
    it is set to false, it will bypass the privilege checking.
    :var name: The name for validation data.
    :var description: The description for validation data.
    :var agentName: The name of the validation agent that this checker belongs to.
    :var category: The name for the category.
    :var closureRule: Struct ClosureRuleInfo describing the closure rule.
    :var attrList: A map of parameter names and parameter values, both name and value is string type.
    :var externalRule: Struct externalRuleInfo describing the external rule.
    :var requirementObject: The object as the requirement for validation.
    """
    existingValidationData: ValidationData = None
    clientId: str = ''
    checkUserPriviledge: bool = False
    name: str = ''
    description: str = ''
    agentName: str = ''
    category: str = ''
    closureRule: ClosureRuleInfo = None
    attrList: List[AttributeInfo] = ()
    externalRule: ExternalRuleInfo = None
    requirementObject: WorkspaceObject = None


@dataclass
class ValidationDataResponse(TcBaseObj):
    """
    The ValidationDataResponse struct holds the response for createOrUpdateValidationData.
    
    :var output: A vector of :ValidationDatasOutput structures.
    :var serviceData: Teamcenter::Soa::Server::ServiceData is the SOA Framework class that holds model objects and
    partial errors. Any created objects and the updated container object will be sent back in the standard ServiceData
    lists of created and updated object respectively. Any failure will be returned with the client ID mapped to the
    error message in the list of partial errors. Following are some possible errors returned:
    - 206001  Insufficient system administration privileges.
    - 206002  Validation data object failed to initialize.
    - 206003  Failed to create validation data objects.
    - 206004  Cannot create duplicate validation data objects.
    - 206005  Failed to get validation data objects from database.
    
    """
    output: List[ValidationDatasOutput] = ()
    serviceData: ServiceData = None


@dataclass
class ValidationDatasOutput(TcBaseObj):
    """
    The ValidationDatasOutput struct contains the output information from createOrUpdateValidationData for result
    validation data and its closure rule and external rule..
    
    :var clientId: Unique client identifier string.
    :var validationDataObject: Pointer to a ValidationData object.
    :var closureRule: Pointer to a ClosureRule object that will be used to collect validation targets from a given
    Business Object.
    :var externalRuleDataset: Pointer to an external rule dataset object that will be used by the ValidationData object.
    :var datasetOutput: A structure that contains the created or updated Dataset and its file ticket information for
    the named references.
    """
    clientId: str = ''
    validationDataObject: ValidationData = None
    closureRule: ClosureRule = None
    externalRuleDataset: Dataset = None
    datasetOutput: DatasetOutput = None


@dataclass
class ValidationResultInfo(TcBaseObj):
    """
    The ValidationResultInfo struct is the main input to the createOrUpdateValidationResults This struct refers to the
    existing ValidationResult, attribute list, report files, and one or more Dataset structs used to create those
    report datasets.
    
    :var existingValidationResult: A valid object pointer to an existing validation result object for update.
    The value will be NULL when a new validation result is to be created.
    :var clientId: Client supplied Identifier string that helps the client map from the input for the object(s) created
    or updated. It should be unique in the client session.
    :var user: The user who performed the validation check and created the validation result.
    :var isAdhoc: A logical flag indicating whether the validation result needs human to decide whether the validation
    result is Passed or Failed.
    :var attrList: A map of parameter name string and parameter value string pairs used for the validation check.
    :var reports: Information to create new validation report datasets that will be attached to the validation result
    object.
    :var replaceExistingReport: This flag specifies whether the new report datasets will replace the existing report
    datasets or will be appended to the existing report datasets array.
    :var targetItemRev: The target ItemRevision of the validation result.
    :var targetObject: The validation target datatset for this validation result.
    :var validationDataObject: The referenced validation data object that was used to create this validation result
    object.
    :var resultId: The value corresponding to the attribute ValidationResult.result_id in the validation result object.
    :var result: This flag indicates whether this validation result is Passed or Failed. When the value is TRUE, the
    result is Passed. When the value is FALSE, the result is Failed.
    :var status: An integer that indicates the validation result status as defined by the validation agent.
    :var comment: The comments for this validation result.
    :var date: The date and time when the validation was last performed.
    """
    existingValidationResult: ValidationResult = None
    clientId: str = ''
    user: User = None
    isAdhoc: bool = False
    attrList: List[AttributeInfo] = ()
    reports: List[ReportFileInfo] = ()
    replaceExistingReport: bool = False
    targetItemRev: ItemRevision = None
    targetObject: Dataset = None
    validationDataObject: ValidationData = None
    resultId: str = ''
    result: bool = False
    status: int = 0
    comment: str = ''
    date: datetime = None


@dataclass
class ValidationResultsOutput(TcBaseObj):
    """
    The structure contains the information for created or updated ValidationResult object and the associated report
    datasets.
    
    :var clientId: Unique client identifier string.
    :var resultObject: Pointer to a ValdiationResult object.
    :var datasetOutputs: A vector of structures that contains the created datasets and the associated file ticket
    information for the named references.
    :var reportDatasets: A vector of updated report datasets associated with the ValidationResult object.
    """
    clientId: str = ''
    resultObject: ValidationResult = None
    datasetOutputs: List[DatasetOutput] = ()
    reportDatasets: List[Dataset] = ()


@dataclass
class ValidationResultsResponse(TcBaseObj):
    """
    Holds the response for createOrUpdateValidationResults.
    
    :var output: A vector of ValidationResultsOutpu structures.
    :var serviceData: Holds model objects and partial errors. Any created objects and the updated container object will
    be sent back in the standard ServiceData lists of created and updated object respectively. Any failure will be
    returned with client id mapped to error message in the ServiceData list of partial errors. Following are some
    possible errors returned in ServiceData:
    - 206002  Validation data object failed to initialize.
    - 206007  Query falied to find the required objects.
    - 206010  Failed to create ValidationResult object.
    
    """
    output: List[ValidationResultsOutput] = ()
    serviceData: ServiceData = None


@dataclass
class ClosureRuleInfo(TcBaseObj):
    """
    The ClosureRuleInfo struct represents the information for a new closure rule or an existing one.
    
    :var useExistingObject: The flag indicates whether to use existing closure rule.
    :var closureRuleObject: Pointer to an existing ClousreRule object when 'useExistingObject' is TRUE.
    :var closureRuleName: The name for a ClosureRule object. If 'useExistingObject' is TRUE and 'closureRuleObject' is
    NULL, a ClosureRule with this name will be used.
    :var type: A vector of DatasetType names. These inputs are only used when 'useExistingObject' is false. They are
    mainly for NX application. When closure rule with the same name does not exist, dataset types will be used to
    create a new closure rule. Otherwise, dataset types will be used to append to current closure rule.
    """
    useExistingObject: bool = False
    closureRuleObject: ClosureRule = None
    closureRuleName: str = ''
    type: List[str] = ()


@dataclass
class CreateOrUpdateValidationAgentsResponse(TcBaseObj):
    """
    The CreateOrUpdateValidationAgentsResponse struct holds the response for createOrUpdateValidationAgents.
    
    :var output: A vector of ValidationAgentOutput structures.
    :var serviceData: Teamcenter::Soa::Server::ServiceData is the  Teamcenter Services framework class that holds model
    objects and partial errors. Any created objects and the updated container object will be sent back in the standard
    ServiceData lists of created and updated object respectively. Any failure will be returned with the client id ID
    mapped to the error message in the ServiceData list of partial errors. Following are some possible errors returned
    in SserviceData:
    - 206001  In sufficient system administration privileges.
    - 206002  Validation data object failed to initialize.
    - 206003  Failed to create validation data objects.
    - 206004  Cannot create duplicate validation data objects.
    - 206005  Failed to get validation data objects from database.
    
    """
    output: List[ValidationAgentOutput] = ()
    serviceData: ServiceData = None


@dataclass
class DatasetFileInfoCAD(TcBaseObj):
    """
    The DatasetFileInfoCAD struct represents all of the data necessary to construct the named references in the dataset
    referenced by validation data and validation result.
    
    :var clientId: Unique client identifier string.
    :var fileName: File name including the path. It cannot be NULL.
    :var namedReferencedName: Name for the named reference. If it is NULL, the reference name in preference
    TC_VALIDATION_FILE_TYPES based on first file extension in the fileinfos will be used.
    :var textFlag: Flag to indicate whether it is binary file or an ASCII text file. Valid values are 0, 1, and 2. 
    0 means it will be decided by system based on reference name, 1 means text file, while 2 means binary file.
    """
    clientId: str = ''
    fileName: str = ''
    namedReferencedName: str = ''
    textFlag: int = 0


@dataclass
class DatasetFileTicketInfoCAD(TcBaseObj):
    """
    The DatasetFileTicketInfoCAD struct contains the information for dataset and its file ticket.
    
    :var datasetFileInfo: The structure contains name reference data, associated file name, file type of a dataset.
    :var ticket: The FMS ticket string of a file for the dataset.
    """
    datasetFileInfo: DatasetFileInfoCAD = None
    ticket: str = ''


@dataclass
class DatasetInfo(TcBaseObj):
    """
    The DatasetInfo struct represents all of the data necessary to construct the dataset object.
    
    :var clientId: Unique client identifier string.
    :var name: The name for a dataset. If NULL, a name will be automatically generated for the new dataset.
    :var type: A dataset type name string. If it is NULL, the type in preference TC_VALIDATION_FILE_TYPES based on the
    first file extension in the fileinfos input parameter will be used.
    :var fileInfos: Information about files attached to the dataset.
    """
    clientId: str = ''
    name: str = ''
    type: str = ''
    fileInfos: List[DatasetFileInfoCAD] = ()


@dataclass
class DatasetOutput(TcBaseObj):
    """
    The DatasetOutput struct contains the created or updated dataset and its file ticket information for the named
    reference.
    
    :var clientId: Unique client identifier string.
    :var dataset: Pointer to a dataset object.
    :var datasetFileTicketInfos: The FMS file ticket info for the dataset.
    """
    clientId: str = ''
    dataset: Dataset = None
    datasetFileTicketInfos: List[DatasetFileTicketInfoCAD] = ()



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject, Item, ItemRevision
from typing import Dict, List
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcBaseObj
from dataclasses import dataclass


@dataclass
class OptionInfo(TcBaseObj):
    """
    This structure contains the value of the for a given launch file option and a flag indicating whether the option
    key name and value should be written to the launch file.
    
    :var optionValue: The value for the given option.
    :var includeInLaunchFile: A flag indicating whether the option key/value pair should also be written into the
    launch file.
    """
    optionValue: str = ''
    includeInLaunchFile: bool = False


@dataclass
class ServerInfo(TcBaseObj):
    """
    This structure holds the basic information for Teamcenter Visualization to connect to the server.
    
    :var protocol: A required parameter referencing the protocol type for connection to the server. Use http for
    standard 4 tier connections, and iiop for 2 tier deployments.
    :var hostpath: A required parameter referencing the URL to connect to the server.
    :var servermode: A required parameter referencing the servermode that controls how the connection to the server is
    made: 2 for two tier. 4 for four tier.
    :var serverAdditionalInfo: An optional parameter referencing the additional information of the server in form of
    key/value pair (if any).
    """
    protocol: str = ''
    hostpath: str = ''
    servermode: int = 0
    serverAdditionalInfo: KeyValueMap = None


@dataclass
class SessionInfo(TcBaseObj):
    """
    This structure holds the information about the session information of the client application from where the launch
    operation was initiated.
    
    :var sessionDescriminator: Client/Server session discriminator to connect to existing specified session.  This
    allows the viewer application to share an existing server process session with the client that initiated the
    launch. If this is not specified, the viewer will present a login prompt.
    :var sessionOptions: Map of option names to OptionInfo structures. These options are used to control the creation
    of the launch information in some way.
    
    Current options include:
    
    "UseTransientVolume" - "True" or "False"
       Determines how the service is to return the launch information. If "True" then the launch information is written
    to a file in the transient volume and a FMS ticket is returned in the ticket field of the LaunchInfoResponce
    structure. If "False" then the launch information is written to a string and that string is placed into the
    vviStrBufferOutputMap field of the LaunchInfoResponce structure. In the later case, the key name to be used is
    found in the "ClientId" option that is now required.
    
    :var sessionAdditionalInfo: An optional parameter referencing the additional information of the session in form of
    key/value pair (if any).
    """
    sessionDescriminator: str = ''
    sessionOptions: OptionsMap = None
    sessionAdditionalInfo: KeyValueMap = None


@dataclass
class UserAgentDataInfo(TcBaseObj):
    """
    This structure holds the information about the client application that initiated the launch.
    
    :var userApplication: An optional parameter referencing the client who initiates the launch.
    :var userAppVersion: An optional parameter referencing the version of the client that initiated launch.
    :var userAdditionalInfo: An optional parameter referencing the additional information of client application in form
    of key/value pair (if any).
    """
    userApplication: str = ''
    userAppVersion: str = ''
    userAdditionalInfo: KeyValueMap = None


@dataclass
class IdInfo(TcBaseObj):
    """
    This structure holds information about the object(s) that will be launched to the viewer. It may contain addtional
    option information that can affect how the VVI is to be generated.
    
    :var launchedObject: The business object to be laucnhed. Launched object could be of type Item, ItemRevision,
    Dataset, BOMView, BOMViewRevision or Awb0ProductContextInfo. The business object will be resolved by the server in
    some cases (e.g. Item or ItemRevision launch) to a directly launchable visualization object (such as a DirectModel
    Dataset or BOMViewRevision).
    :var item: The parent or containing Item of the launched object.  If this is not provided, the server will attempt
    to identify the parent if it can. If the parent information cannot be identified, this will not result in an error
    condition. The parent Item information will simply not be passed to the visualization client, which could affect
    what features are available in the client.
    :var itemRev: The parent or containing ItemRevision of the launched object. If this is not known, the server will
    attempt to identify the parent if it can. If the parent information cannot be identified, this will not result in
    an error condition. The parent ItemRevision information will simply not be passed to the visualization client,
    which could affect what features are available in the client.
    :var occurrencesList: The list of selected occurrences being launched. These occurrences may be of type BOMLine or
    Awb0Element. When this vector is non-empty then the launchedObject field of the IdInfo structure must contain an
    object that defines the configuration state for the occurrences. For example, the configuration object is of type
    Awb0ProductContextInfo when the occurrences are of type Awb0Element. The configuration object is of type BOMWindow
    when the occurrences are of type BOMLine.
    :var createOptions: Map of option names to OptionInfo structures. These options are used to control the creation of
    the launch information in some way.
    
    Current options include:
    
    "CreateVisSC" - "True" or "False"
       Informs the service to create a VisStructureContext object based on the laucnhedObject and occurrenceList field
    of the IdInfo structure.
    
    "OVERRIDE_VisDoc_Type" - some replacement string
       Directs the service to replace the value of the VisDoc_Type key in the launch file with the specified string.
    
    "TransientDoc" - "True" or "False"
       Indicates whether the launched object is to be considered transient by the viewer. Transient objects are
    normally deleted when the viewer is finished with the document used to open the launched object.
    
    "Operation" - "open", "insert", "merge", "interop"
       Informs the viewer on what type of launch operation is being requested.
    
    "OperationStructure" - "Dynamic", "Static", "Preference", "Ask"
       Informs the viewer on what type of structure launch is being requested.
    
    "UseTransientVolume" - "True" or "False"
       Determines how the service is to return the launch information. If "True" then the launch information is written
    to a file in the transient volume and a FMS ticket is returned in the ticket field of the LaunchInfoResponce
    structure. If "False" then the launch information is written to a string and that string is placed into the
    vviStrBufferOutputMap field of the LaunchInfoResponce structure. In the later case, the key name to be used is
    found in the "ClientId" option that is now required.
    
    "ClientId" - a unique string used to identify this call to the service.
       Is a required option when the "TransientVolume" option is "False".
    
    "Client" - string used to identify the client calling this service
        Currently the only value being recognized is "AW", which indicates that ActiveWorkspace is the desired client
    viewer.
    
    
    :var idAdditionalInfo: This is a generic mechanism for putting additional key/value pairs into the output launch
    information. This data is considered opaque to the service and the key/value pairs are simply output into the VVI
    launch information.
    """
    launchedObject: BusinessObject = None
    item: Item = None
    itemRev: ItemRevision = None
    occurrencesList: List[BusinessObject] = ()
    createOptions: OptionsMap = None
    idAdditionalInfo: KeyValueMap = None


@dataclass
class LaunchInfoResponse(TcBaseObj):
    """
    The output response structure for the createLaunchInfo () operation.
    
    :var ticket: Holds a FMS transient file ticket for the launch information written to the VVI file.
    This member is valid only when the "UseTransientVolume" option from the IdInfo input structure was set to "True".
    :var vviStrBuffersOutputMap: The map of clientId string to the VVI launch information string buffer(string/string).
    This member is valid only when the "UseTransientVolume" option from the IdInfo input structure was set to "False".
    Also requires the "ClientId" option from the IdInfo input structure to be present and set to non-zero length
    string. The value of the "ClientId" option is used as the key to this map.
    :var serviceData: SOA Framework object containing error information. In cases where objects cannot be launched,
    error message, codes will be mapped to respective object id in the list of partial errors.
    """
    ticket: str = ''
    vviStrBuffersOutputMap: VVIStringBufferOutputMap = None
    serviceData: ServiceData = None


"""
Map of key-value name pairs, each of type std::string.
"""
KeyValueMap = Dict[str, str]


"""
A map of option key names to OptionInfo structures. These options are used to control the creation of the launch file in some way. 

"""
OptionsMap = Dict[str, OptionInfo]


"""
A map containing client ids or additional Teamcenter object UID  and vviStringBuffer as key/value pairs.
"""
VVIStringBufferOutputMap = Dict[str, str]



from __future__ import annotations

from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcBaseObj
from dataclasses import dataclass
from tcsoa.gen.BusinessObjects import BusinessObject
from typing import Dict


@dataclass
class ClassifyCommandVisibilityInfoResp(TcBaseObj):
    """
    Structure representing the classification visibility information returned by getClassificationCmdVisibilityInfo
    operation.
    
    :var wso2ClassifyMap: A map (WorkspaceObject, bool) of workspace objects to corresponding boolean visibility value.
    True &ndash; WorkspaceObject is classifiable. "Add" command will be visible in Classification tab.
    False &ndash; WorkspaceObject is not classifiable. "Add" command will not be visible in Classification tab.
    :var serviceData: Any failures will be returned in the service data list of partial errors.
    """
    wso2ClassifyMap: Wso2ClassifyMap = None
    serviceData: ServiceData = None


"""
Map of WorkspaceObject to corresponding Boolean visibility value.
"""
Wso2ClassifyMap = Dict[BusinessObject, bool]



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject, Fnd0ModelViewProxy
from typing import Dict, List
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcBaseObj
from dataclasses import dataclass


@dataclass
class FindModelViewsInStructureResponse(TcBaseObj):
    """
    The response contains any found model view proxies ( FndModelViewProxy) and the structure location at which they
    were found. Also, if the "compareWithMVList" option is set to true, then an additional list of model view proxies
    that are currently disclosed but appear to no longer be relevant is returned.
    Any errors are returned in the 'serviceData'.
    
    :var modelViewsByStructureNodes: A list of proxy objects found for various nodes in the structure - the structure
    nodes may be bomlines, item revisions or design elements depending on the type of structure starting scope and
    disclosure type.
    :var unfoundFromModelViewList: The model view proxy objects (Fnd0ModelViewProxy) that are currently associated to
    the specified disclosure but are not found in the given 'startingScope'. Note that if the 'startingScope' is not
    the disclosure, that the missing proxy objects may still be present in a disclosure elsewhere in its structure.
    This list is only populated if the compareWithMVList option is set to true.
    :var possibleMatching: Map (Fnd0ModelViewProxy, Fnd0ModelViewProxy) of possible matching proxies that are
    equivalent. The keys are proxies in an associated MVList (associated to the input structure object or disclosure)
    that are no longer precisely in the structure with the given configuration. The values are the system suggested new
    proxies that are equivalent. This map is only populated if compareWithMVList option is set to true.
    :var serviceData: Contains a list of any errors which occurred during the operation.
    """
    modelViewsByStructureNodes: List[StructureNodeResults] = ()
    unfoundFromModelViewList: List[Fnd0ModelViewProxy] = ()
    possibleMatching: BoToBoMap = None
    serviceData: ServiceData = None


@dataclass
class StructureNodeResults(TcBaseObj):
    """
    Identifies a list (containing at least one) of model view proxies (Fnd0ModelViewProxy) that were found for a
    particular structure node. Assisting context information ('csIdContextOccurrence') may be provided to help locate
    the structure node.
    
    :var structureNode: A node of the structure for which model views have been found. The structure node could either
    be a BOMLine or for some cases of Workset structure, a model element (Mdl0ModelElement).
    :var resultingViews: List of model view proxies (Fnd0ModelViewProxy) found for the particular structure node.
    :var csIdContextOccurrence: Contains either the subset in which the structure node resides (either Cpd0SubsetLine
    or Cpd0DesignSubsetElement.) or the 'startingScope' ItemRevision or BOMLine if multiple 'startingScope' values were
    provided. This object gives context to the associated 'clonestableIdChain' value(s).
    :var clonestableId: A list of one or more clone stable occurrence id values used to help locate the 'structureNode'
    within the 'csIdContextoccurrence'. The clone stable occurrence id values are may be ordered from a given
    'startingScope' - which is repeated in the 'StructureNodeResults' as the 'csIdContextOccurrence'. 
    
    Alternatively, if a Workset is given as a 'startingScope', the 'csIdContextOccurrence' may be a subset in the
    Workset, and only a single cs_id value for the 'structureNode' would be needed since a given model element
    (Mdl0ModelElement) in a subset (Cpd0DesignSubsetElement) can be identified by such a single clone stable id.
    
    Also, if the 'resultingViews' are found directly off a 'startingScope', then the 'csIdContextOccurrence' and
    'structureNode' will both be the 'startingScope' and the 'clonestableIdChain' will be empty as no expansion was
    used to find the resultingViews.
    """
    structureNode: BusinessObject = None
    resultingViews: List[BusinessObject] = ()
    csIdContextOccurrence: BusinessObject = None
    clonestableId: List[str] = ()


"""
Generic map of one business object to another.
"""
BoToBoMap = Dict[BusinessObject, BusinessObject]


"""
Generic map of boolean names to values (string, bool). Commonly used to contain selected options from a client.
"""
BoolMap = Dict[str, bool]



from __future__ import annotations

from tcsoa.gen.Administration._2012_09.PreferenceManagement import PreferenceResponseWithFileTicket, ImportPreferencesAtLocationsIn, ImportPreferencesAtLocationDryRunResponse, GetPreferencesResponse, SetPreferences2In, PreferencesAtLocationIn, ImportPreferencesAtLocationDryRunIn, PreferenceLocation, SetPreferencesDefinitionIn, SetPreferencesAtLocationsIn, GetPreferencesAtLocationsResponse
from tcsoa.gen.Administration._2012_09.UserManagement import GroupMemberInput, GetUserGroupMembersInputData, GetUserGroupMembersResponse
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcService


class UserManagementService(TcService):

    @classmethod
    def getUserGroupMembers(cls, inputObjects: List[GetUserGroupMembersInputData]) -> GetUserGroupMembersResponse:
        """
        This operation retrieves information of all group members for a list of users specified in the list of
        GetUserGroupMembersInputData inputs. The information includes Group object, Role object, User object, status,
        group admin privilege, and default role flag of the user group members. The returned results could contain
        information only for the active group members of the user or both active and inactive group members of the user
        depending on option includeInactive setting in GetUserGroupMembersInputData.
        
        Exceptions:
        >If system errors occur.
        """
        return cls.execute_soa_method(
            method_name='getUserGroupMembers',
            library='Administration',
            service_date='2012_09',
            service_name='UserManagement',
            params={'inputObjects': inputObjects},
            response_cls=GetUserGroupMembersResponse,
        )

    @classmethod
    def setGroupMemberProperties(cls, inputObjects: List[GroupMemberInput]) -> ServiceData:
        """
        This operation updates the properties on one or more GroupMembers.The following properties may be updated:
        membership_data_source, ga,default_role, status.
        """
        return cls.execute_soa_method(
            method_name='setGroupMemberProperties',
            library='Administration',
            service_date='2012_09',
            service_name='UserManagement',
            params={'inputObjects': inputObjects},
            response_cls=ServiceData,
        )


class PreferenceManagementService(TcService):

    @classmethod
    def importPreferencesAtLocationDryRun(cls, importPreferences: ImportPreferencesAtLocationDryRunIn) -> ImportPreferencesAtLocationDryRunResponse:
        """
        Pretends to import the preferences from the input file into the specified location.
        On the contrary to the import operation, the dry run operates on one location at a time.
        The objective is to gather information on what would be the final result for proceeding with the real import
        operation.
        
        This operation takes a vector of structure representing the preferences and the location where to import.
        The valid values for the location parameter in the PreferenceLocation structure are: Site, Group, Role, User.
        The Site value means that the preference will be imported for the entire organization.
        Group or Role means that the value will be imported for the Group or Role of the current logged-in user.
        It is also possible to import for the non-current user.
        """
        return cls.execute_soa_method(
            method_name='importPreferencesAtLocationDryRun',
            library='Administration',
            service_date='2012_09',
            service_name='PreferenceManagement',
            params={'importPreferences': importPreferences},
            response_cls=ImportPreferencesAtLocationDryRunResponse,
        )

    @classmethod
    def importPreferencesAtLocations(cls, importPreferenceIn: ImportPreferencesAtLocationsIn) -> PreferenceResponseWithFileTicket:
        """
        Imports the preferences from the input file into the specified locations.
        
        This operation takes a vector of structure representing the preferences and the location where to import.
        The valid values for the location parameter in the PreferenceLocation structure are: Site, Group, Role, User.
        The Site value means that the preference will be imported for the entire organization.
        Group or Role means that the value will be imported for the Group or Role of the current logged-in user.
        It is also possible to import for the non-current user.
        """
        return cls.execute_soa_method(
            method_name='importPreferencesAtLocations',
            library='Administration',
            service_date='2012_09',
            service_name='PreferenceManagement',
            params={'importPreferenceIn': importPreferenceIn},
            response_cls=PreferenceResponseWithFileTicket,
        )

    @classmethod
    def removeStalePreferenceInstancesAtLocations(cls, locations: List[PreferenceLocation]) -> ServiceData:
        """
        Since the preference manager utility has a cleanup mode that does the same thing as
        removeStalePreferenceInstancesAtLocations-_2012_09-PreferenceManagementService. There is no need of a SOA
        operation to achieve this.
        """
        return cls.execute_soa_method(
            method_name='removeStalePreferenceInstancesAtLocations',
            library='Administration',
            service_date='2012_09',
            service_name='PreferenceManagement',
            params={'locations': locations},
            response_cls=ServiceData,
        )

    @classmethod
    def setPreferences2(cls, preferenceInput: List[SetPreferences2In]) -> ServiceData:
        """
        Allows the logged-in user to set the values for the given preferences for the logged-in user.
        Values can only be given to preferences already defined in the system and for which the protection scope allows
        the user to give a value. Otherwise, the operation will return an error for this preference.
        Preference name and values are specified in input structure 'SetPreferences2In'. 
        """
        return cls.execute_soa_method(
            method_name='setPreferences2',
            library='Administration',
            service_date='2012_09',
            service_name='PreferenceManagement',
            params={'preferenceInput': preferenceInput},
            response_cls=ServiceData,
        )

    @classmethod
    def setPreferencesAtLocations(cls, setPreferenceIn: List[SetPreferencesAtLocationsIn]) -> ServiceData:
        """
        Sets the values for the specified preferences and locations. <br/><br/>The input 'PreferenceLocation' structure
        (within the 'SetPreferencesAtLocationsIn' structure) contains 2 mutually exclusive parameters: <ol><li>The
        location parameter, for which values can be for this specific operation: <ul type="circle"><li>"Site": The
        preference will be set for the entire organization.
        <li>"Group": the preference will be set for the group of the logged-in user. 
        <li>"Role": The preference will be set for the role of the logged-in user. 
        <li>"User": The preference will be set for the logged-in user.
        </ul>
        <li> The object parameter, which represents the User, Role or Group where to set the values. This is to be used
        when the target is for the non-logged-in user. Note that the caller must have permission to set the preference
        values for the specified object. 
        </ol>
        """
        return cls.execute_soa_method(
            method_name='setPreferencesAtLocations',
            library='Administration',
            service_date='2012_09',
            service_name='PreferenceManagement',
            params={'setPreferenceIn': setPreferenceIn},
            response_cls=ServiceData,
        )

    @classmethod
    def setPreferencesDefinition(cls, preferenceInput: List[SetPreferencesDefinitionIn]) -> ServiceData:
        """
        Allows system administrators to create the definitions for new preferences, or to modify existing preference
        definitions. If the preferences do not exist, it sets the definitions and values for the specified preferences.
        If the preferences already exist (i.e. they have been defined already), it modifies them. 
        
        This is a system administrator level operation. The intent is that only the system administrator should create
        a preference. However, this could be needed from non-directly-related user interactions. Therefore, the
        decision to make a call to this operation is delegated to its caller. This operation takes a list of
        'SetPreferencesDefinitionIn' structures, which contains a 'PreferenceDefinition' structure. This structure can
        be simplified in case of preference modifications. 
        
        Its parameters are: 
        
        - category: The category where the preference is stored. If the input string is empty, the parameter will not
        be taken into account. However, in case the preference has not been created yet, it will be assumed that the
        preference will go under the "General" category. If the string is not empty, and if the category does not
        exists in the system already, new category would be created and the preference would go under this category.  
        
        
        
        - description: The textual explanation of what the preference does. If the input string is empty, the parameter
        will not be taken into account. 
        
        
        
        - type: The preference type. Valid values are:
        
        
                                0: String preference. 
                                1: Logical preference. 
                                2: Integer preference. 
                                3: Double preference. 
                                4: Date preference. 
                        If the preference does not exist, this piece of information will be needed. If the preference
        exists and if the value is provided and if a preference instance already exist with the old type and the
        conversion from the old type to the new one is not possible and an error would be returned. 
        
        - protectionScope: The level at which the preference is protected. Valid values are:                        
        
        
                                "User": All users can provide a value for the preference. 
                                "Role": Only system and group administrators can provide a value. 
                                "Group": Only system and group administrators can provide a value. 
                                "Site": Only system administrators can provide a value. 
                                "System": Only system administrators can provide a value. Furthermore, the protection
        scope cannot be changed from then on. If the preference does not exist, this information is mandatory. If the
        preference exists and the protection scope string is empty, the parameter will not be taken into account. If
        the  preference exists and the protection scope string is not empty, the code will consider this to be a
        modification. 
        - isEnvEnabled: Status if the preference value can be set through an environment variable, in which case it
        will come from that location first. This piece of information will always be taken into consideration.
        
        
        
        - isOOTBPreference: Not used for this operation.
        
        """
        return cls.execute_soa_method(
            method_name='setPreferencesDefinition',
            library='Administration',
            service_date='2012_09',
            service_name='PreferenceManagement',
            params={'preferenceInput': preferenceInput},
            response_cls=ServiceData,
        )

    @classmethod
    def deletePreferenceDefinitions(cls, preferenceNames: List[str], deleteAllCustomDefinitions: bool) -> ServiceData:
        """
        Deletes the definition and all value instances of the specified preferences.
        Since preferences will not be differentiated as Custom or COTS going forward, the input parameter
        deleteAllCustomDefinitions will not be used when performing this operation.
        """
        return cls.execute_soa_method(
            method_name='deletePreferenceDefinitions',
            library='Administration',
            service_date='2012_09',
            service_name='PreferenceManagement',
            params={'preferenceNames': preferenceNames, 'deleteAllCustomDefinitions': deleteAllCustomDefinitions},
            response_cls=ServiceData,
        )

    @classmethod
    def deletePreferencesAtLocations(cls, deletePreferencesAtLocationIn: List[PreferencesAtLocationIn]) -> ServiceData:
        """
        Deletes the given preference instances at the specified location only. 
        This operation takes a list of 'PreferencesAtLocationIn' structures representing the preferences to delete and
        the location. 
        The location is a 'PreferenceLocation' structure, which contains 2 mutually exclusive parameters:
        1. The location parameter, for which values can be for this specific operation:
        - "Site": The preference will be deleted from current site location.
        - "Group": The preference will be deleted for the group of the logged-in user.
        - "Role": The preference will be deleted for the role of the logged-in user.
        - "User": The preference will be deleted for the logged-in user.
        
        
        2. The object parameter, which represents the User, Role or Group where to delete the values. This is to be
        used when the target is for the non-logged-in user.
        
        Use cases:
        A preference instance is present at the user level, and it needs to be removed.
        """
        return cls.execute_soa_method(
            method_name='deletePreferencesAtLocations',
            library='Administration',
            service_date='2012_09',
            service_name='PreferenceManagement',
            params={'deletePreferencesAtLocationIn': deletePreferencesAtLocationIn},
            response_cls=ServiceData,
        )

    @classmethod
    def getPreferences(cls, preferenceNames: List[str], includePreferenceDescriptions: bool) -> GetPreferencesResponse:
        """
        Retrieves the values for the preferences specified in the list of names, as seen by the current logged-in user,
        based on current application context. If there are no values in current application context, values are
        retrieved from default application context if exists.
        If the list is empty or its first value is equal to "*", all the preferences as seen by the logged-in user will
        be returned (not only the preference instances created by the logged-in user).
        
        Use cases:
        Retrieving the value for a series of preferences.
        """
        return cls.execute_soa_method(
            method_name='getPreferences',
            library='Administration',
            service_date='2012_09',
            service_name='PreferenceManagement',
            params={'preferenceNames': preferenceNames, 'includePreferenceDescriptions': includePreferenceDescriptions},
            response_cls=GetPreferencesResponse,
        )

    @classmethod
    def getPreferencesAtLocations(cls, getPreferenceAtLocationIn: List[PreferencesAtLocationIn], includePreferenceDescriptions: bool) -> GetPreferencesAtLocationsResponse:
        """
        Retrieves the values for the specified preferences and locations.<br/>The input structure contains:<ul><li>A
        vector preference names. If the list is empty or its first element is "*", all the preferences for the
        specified locations (and only for preference instances at this location) are being returned.
        <li>A <font face="Courier" height="10">PreferenceLocation'  structure, which contains 2 mutually exclusive
        parameters:<ol><li>The location parameter, for which values can be for this specific operation:<ul
        type="circle"><li>"Site": The preference will be retrieved either as overwritten or as an out-of-the-box (OOTB)
        value (whichever gives a value first). 
        <li>"Group": The preference will be retrieved for the group of the logged-in user. 
        <li>"Role": The preference will be retrieved for the role of the logged-in user. 
        <li>"User": The preference will be retrieved for the logged-in user.
        </ul>
        <li> The object parameter, which represents the User, Role or Group where to retrieve the values. This is to be
        used when the target is for the non-logged-in user.
        </ol>
        </ul>
        
        Use cases:
        1. The logged-in user needs to know the preference value given at her/his Role or Group level.
        2. The logged-in user needs to know the preference value given by another user, or a Role/Group than hers/his.
        """
        return cls.execute_soa_method(
            method_name='getPreferencesAtLocations',
            library='Administration',
            service_date='2012_09',
            service_name='PreferenceManagement',
            params={'getPreferenceAtLocationIn': getPreferenceAtLocationIn, 'includePreferenceDescriptions': includePreferenceDescriptions},
            response_cls=GetPreferencesAtLocationsResponse,
        )
